% function Decomp15
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Decomp
%   Separation into incident and reflected wave
%   input: data file, wave heights measured by 2 or 3 wave 
%          gauges; more wave gauges allowed, not yet tested
%          parameter files: 2 files, described below
%   version 15: Jan 25, 2005, input and output in general functions
%       31-Mar-2005, resulting f-range from user input
%       and threshold compared with f-range of ideal Jonswap sp.
%
%   autheur Henk Jan Bakkenes, Delft 2002
%   completed by Hetty Klaasman, 2003-2005
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   version 15: Jan 25, 2005, input and output in general functions
%       31-Mar-2005, resulting f-range from user input
%       and threshold compared with f-range of ideal Jonswap sp.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   input files:
%   data file:  Ascii, header allowed, fixed number of chanels,
%               separated by tabs, spaces, commas or ';'.
%   parameter file (format described in manual and in FN_inparmd3):
%               time step, approximated peak period Tp, 
%               freq step/Tp, 
%               maximum freq-range (if f_max=-1: calculated in case of
%               2 wave gauges, and replaced by [0.02,2.5] in case of 3 
%               wave gauges or more,
%               thresh (E>=thresh*Emax), 
%               column numbers and scale of wave gauges
%   file containing depths as a function of the position:
%               pos, depth in at least P+1 points, P number of wave
%               gauges.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   frequecy ranges used by this program:
%   f_min, f_max:   defined in parameter file or calculated/replaced
%                   by default if f_max=-1
%   fmin_s, fmax_s: range where spectrum above threshold,
%   fmin_r, fmax_r: range where spectral moments of measurements are 
%                   calculated,
%   fmin_cr, fmax_cr:
%                   range where the decomposition will be calculated
%                   and displayed in graphs,
%   fmin_t, fmax_t: range where spectral moments are calculated.
%
%   the variance spectrum E_i of the incident wave is calculated
%   in a greater freq.range than the valid freqs, because this 
%   will enhance the interpretation of the plot;
%   freq-range of ideal Jonswap Sectrum at given Tp, thresh will be
%   checked
%
%   Method: Matrix decomposition, described in Msc-thesis of Henk Jan
%   Bakkenes (June 2002), pp 22-24
%   See also: Zelt and Skjelbreia (1992), Estimating Incident and 
%       Reflected Wave Fields Using an Arbitrary Number of Wave Gauges, 
%       Proc. 23rd Int. Conf. Coastal Eng., ASCE, pp. 777-789
%       (Wj,p taken 1)
%
%   In case of 2 wave gauges and f_max=-1, f_min and f_max are 
%   calculated from Goda (1985), p. 302
%   delta_l/Lmax = 0.05,  delta_l/Lmin = 0.45
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all;
%addpath('D:\Hetty\Matlab\Lab_process\Functions');
addpath('.\Functions');
err1 = 0;

% =========================================================================
% ========= INPUT =========================================================
% =========================================================================

% read output desired:
[outs, start2] = FN_chsoutp;
if start2==1
    % read Ascii file (e.g. Dasylab file)
    % file name and other specifications are asked from screen
    [in_naam, pad_naam, Fileinfo, A, type, err1] = FN_infile8;

    if err1==0
    
        % read parameter file and depth profile:
        scr_num = 7;
        [nsamp,nchan] = size(A);
        [Inparm,err1] = FN_inparm2(scr_num,nchan);
        
        if err1==0
            % check frequency 
            
            % select part of A to be analysed:
            show = 1;
            [Scanparm,Ts,A1,err1] = FN_selectnw2(A,Fileinfo,Inparm,show);
            clear A;

            if err1==0
    
               hw = waitbar(0,'calculating ....');
               % calculate statistical moments of time signal
                [A2,Outparmts] = FN_momts(A1,Inparm,Scanparm);
                clear A1;
            
                % calculate raw FFT-spectra (to be used in the decomposition), 
                % variance spectra averaged over freq. bands, and spectral
                % parameters from raw spectra:
                waitbar(0.75);
                close(hw);
                [F,A_jjpp,Outparm] = ...
                    FN_spD(in_naam,Ts,A2,Inparm,Scanparm,outs);
                 
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                % compare selected freq-range with ideal Jonswap spectrum
                % of same TpD (=peak period above 80% of energy);
                % and warn if this range is outside [f_min,f_max]
                % specified by the user (in Outparm, because f_max may be -1):
                % moved: will be executed after the calculation
                % because the msgbox must be removed after clicking OK;
                % if you want to change the freq-range from the
                % results of FN_jonswap, move the lines to here.
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                
                % copy stat. moments to Outparm, because the general
                % print function uses 1 output parameter Outparm:
                Outparm.M1    = Outparmts.M1; %  mean value is 1st moment 
                Outparm.var   = Outparmts.var; %  variance of time signal
                Outparm.Hm0t  = Outparmts.Hm0t; %4.*(Outparmts.var .^0.5)
                
                % All next output parameters must be copied to Outparm !!
                    
                % Decomposition:
                [y_inc,y_rfL,Fs,Es_i,Es_r,Outref,err1] = ...
                    FN_decomp(F,A_jjpp,Inparm,Scanparm,Outparm,outs);
                   
                % copy results to Outparm:        
                Outparm.fmax_cr = Outref.fmax_cr;
                Outparm.fmin_cr = Outref.fmin_cr;
                Outparm.fmax_cr = Outref.fmax_cr;
                Outparm.Kr      = Outref.Kr;
                Outparm.m0i     = Outref.m0i;
                Outparm.m0r     = Outref.m0r;
                % Outparm.Hm0     = Outref.Hm0;
                % Outparm.Hm0m    = Outref.Hm0m;
                Outparm.Hm0i    = Outref.Hm0i;
                Outparm.Hm0r    = Outref.Hm0r;
                Outparm.sumi    = Outref.sumi;
                Outparm.sumr    = Outref.sumr;
                Outparm.Tpim    = Outref.Tpim;
                Outparm.fpim    = Outref.fpim;
                Outparm.Epim    = Outref.Epim;
                
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                % compare selected freq-range with ideal Jonswap spectrum
                % of same TpD (=peak period above 80% of energy);
                % and warn if this range is outside [f_min,f_max]
                % specified by the user (in Outparm, because f_max may be
                % -1);
                % executed after the calculation because the msgbox must be 
                % removed after clicking OK;
                % if you want to change the freq-range from the
                % results of FN_jonswap, move up the lines below to the line
                % following FN_spD.
                Tpm     = mean(Outparm.Tpm);
                thresh  = Inparm.thresh;
                % version 15:
                %  f_min   = Outparm.f_min;
                %  f_max   = Outparm.f_max;
                
                % changed in version 16:
                f_min   = Outparm.fmin_t;
                f_max   = Outparm.fmax_t;
                [Outparm.fmin_js, Outparm.fmax_js] = ...
                    FN_jonswap(Tpm(2),thresh,f_min,f_max);
                
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                              
                                
                % plot of filtered spectra:
                if outs(3)==1 || outs(4)==1
                    [nsamp,nchan] = size(A2);
                    interp  = 0;
                    method  = 1;
                    % calculate figure number; the spectra from the wave
                    % gauges have been plotted in FN_spD, 
                    % depending from outs(1) and outs(2):
                    nfig    = (outs(1)+outs(2))*nchan + 1;
                    
                    FN_plotD16(nfig,in_naam,Inparm,Outparm,Fs',Es_i', ...
                        Es_r',interp,method,outs);
                end        

                % results of calculations:
                if outs(5)==1
                    scr_num = 7;
                    FN_outparm(scr_num,in_naam,Inparm,Scanparm,Outparm,...
                        err1);
                end    
                % spectra:
                if outs(6)==1
                    % de bedoeling was deze te vervangen door FN_outdata2
                    % zie alle wijzigingen in Inparm van 15 of 16-12
                    out_num = 2;
                    FN_outdata3(out_num,in_naam,Inparm,Scanparm,Outparm,...
                        Fs,Es_i,Es_r);
                    %FN_outDsp(in_naam,Inparm,x,h,xp,fs,Es_i,Es_o,Outparm);
                end    

                        
                % write results to outputfile:
                % ook vervangen door FN_Outadata2, zie boven
                if outs(7)==1
                    out_num = 1;
                    FN_outdata3(out_num,in_naam,Inparm,Scanparm,Outparm,...
                        Ts,y_inc,y_rfL);
                end %outs(7)
                
            end     % no errors in selection
        end     % no errors in input
    end     % data read
end     % start2==1, output indicated by user    

% Error handling
FN_error(err1);

% end Decomp15