function [y_inc,y_rfL,Fs,Es_i,Es_r,Outref,error] = ...
         FN_Decomp(F,A_jjpp,Inparm,Scanparm,Outparmsp,outs);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% FN_Decomp
%   Separation into incident and reflected wave
%   version 1
%   1-Feb-05
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   input:
%   F:      frequency vector of the raw spectrum
%   A_jjpp: matrix of FFT-values per chan.
%	Inparm:	structure met invoerparameters
%       dt:     tijdstap
%       fres:   gewenste frequentieresolutie in variantiespectrum
%       f_max:  bovengrens te berekenen freq.gebied
%       thresh: voor Spect: drempel voor het te plotten deel van het spectrum
%       colt:   kanaal met de tijd (0 voor geen)
%       ncols:  aantal te verwerken kanalen
%	    col:	rijvector met kolomnummers kanalen
%       xp:     rijvector met X-coordinaten (xp: xpos, n.a.v. Decomp v.14 en ouder)
%       hxp:    dieptes op plaatsen van de golfhoogtemeters (wordt berekend
%               uit overige invoerparameters, alleen Decomp)
%       x:      rijvector met x-coordinaten waarvoor het diepteprofiel is
%               gegeven
%       h:      dieptes in diepteprofiel
%   Scanparm:   structure gegevens over met gekozen samples:
%       tin1:   gevraagde begintijd 
%       tin2:   gevraagde eindtijd
%       step:   gevraagde monsterstap bij selectie
%       tuit1:  begintijd na bemonsteren
%       tuit2:  eindtijd na bemonsteren
%       dt:     tijdstap na bemonsteren
%   Outparmsp:  structure with spectral parameters
%               (calculated from raw spectra)
%       N:      number of samples used in analyses     
%       df:     frequency step in raw spectra
%       fres:   frequency resolution used to calulate av. spectra
%       sp_sum: 1: spectra will be summed over bands,
%               0: Inparm.fres<df, no summation, fres set to df
%       fres:   frequency resolution used to calulate av. spectra
%       Tpm:    vector of peak periods
%       fpm:    vector of peak frequencies
%       Epm:    variance density at peak frequency
%       f_min, f_max:   nog nakijken
%       fmin_s, fmax_s: connected region of E>=thresh*Emax
%       fmin_r, fmax_r: region of calculation spectral moments (combination
%                       of f_min(max) and fmin(max)_s)
%       m:      matrix of spectral moments, m(i,j) = ith moment of ch j
%       T0:     matrix of peak periods, T(1,j): Tp of ch j, 
%               T(2,j): T80% of ch j
%       eps, kappa, gamma: combinations of moments, described in manual
%       Hm0     = 4.*(m(1,:).^0.5);
%   outs:   output indicator: outs(7)=1: reconstruction of inc. and refl. wave
%           outs(7)=0: no reconstruction    
%
%   output:
%   y_inc:  incident wave (time signal) if outs(7)=1, otherwise []
%   y_rfL:  reflected wave (time signal) if outs(7)=0, otherwise []
%   Fs:     frequecy vector of average spectra E_i, E_o
%   Es_i:   variance spectrum of incident wave, averaged over freq.bands
%   Es_r:   variance spectrum of reflected wave, averaged over freq.bands
%
%   Outref: structure with spectral parameters calculated after decomposition
%       fmin_cr, fmax_cr:
%               freq. range where the decomposition will be calculated
%               and displayed in graphs,
%       Kr:     terugkaatsingscoef. over gebied [f_min,f_max]
%       m0i:    mo of incident wave  
%       m0r:    mo of reflected wave
%       Hm0:    vector with significant waveheights per wave gauge
%       Hm0m:   significant wave height, averaged over wave gauges
%       Hm0i:   significante wave height of incident wave
%       Hm0r:   significante wave height of reflected wave
%       sumi:   sum of variances of incident wave
%       sumr:   sum of variances of reflected wave
%       Tpim, fpim, Epim: ppeak period, peak freq., and max. vlue of Ei 
%   error:  error detection: 0 no errors, >0 error
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Initialize:
y_inc       = [];
y_rfL       = [];
Fs          = F;
E_i         = A_jjpp;
[nf,nchan]  = size(A_jjpp);
E_r         = zeros(nf,nchan);
Outref.K    = 0;
        
% bandwidth in avaraged variance spectra:
df      = Outparmsp.df;
fres    = Outparmsp.fres;  % used frequency resolution
thresh  = Inparm.thresh;
      
% standard settings:
g           = 9.81;				% g : gravity acceleration [m/s2]
no_gauges   = nchan;
N           = Outparmsp.N;    % number of dtapoints (mogelijk niet nodig)
Efact       = 0.5*N*Scanparm.dt; % scale factor of the variance spectrum, 
                                 % see FN_spD
                    
% freqency range of the plotted part of the spectrum (this part of the 
% spectrum will be used in the calculation of decomposition.
% Note: at frequencies where the distance of the wave gauges is near the 
% half wave length, the system of equations, will become bad conditioned, 
% which results in a local peak in the spectrum. If f_max is according to 
% the distance of the wave gauges, the peak will be near f_max. For that 
% reason, the upper limit must not be far from f_max.
                    
% use one of the next formulas if more frequecies are to be inspected:
% fmin_cr = min(f_min,fmin_s); 
% fmin_cr = 0.95*fmin_cr;
% or:
% fmin_cr = df;
% fmax_cr = fmax_s;
fmin_t  = Outparmsp.fmin_t;
fmax_t  = Outparmsp.fmax_t;
fmin_cr = 0.95*fmin_t;
fmax_cr = 1.05*fmax_t;
                    
% Decomposition:
                    
% =================================================================================
% ====== Calculations shoaling and wave number free waves === PHASE_INT.m =========
% =================================================================================
f_LF    = [df:df:fmax_cr]';  	% f_LF : LF frequency vector
K       = phase_int(Inparm.xp,f_LF,Inparm.x,Inparm.h);  
                                % determination of wave phase and shoaling

a_res = zeros(length(f_LF),2); % initialisation
A_jjpp(length(f_LF)+1:size(A_jjpp,1),:) = [];  % Only necessary for f_LF  %%
%
% ========== Decomposition calculations ==========================================
error   = 0;
jj      = round(fmin_cr/df);
nf_s    = length(f_LF);
% for jj = round(fmin_cr/df):length(f_LF)
% for replaced by while to be able to stop if condition is not met, 
%  Hkl, 26-4-04
while jj<=nf_s & error<300        
    kk = 0;
    for pp = 1:no_gauges
        kk = kk+1;
        B(kk,:) = [K(jj,pp) conj(K(jj,pp))];
    end
    % condition number of matrix B: check singularities 
    % --> for adjustment of gauges distances or f_limits
    try
        condit(jj,1) = cond(B);%%
    catch
        Le   = lasterr;
        if length(Le)>0
            error = 305;
        end    
    end
    if error==0
        % According to help-file Matlab uses QR-factorization 
        % to solve overdetermined systems
        A = A_jjpp(jj,1:no_gauges).';
        a(jj,:) = [B\A].';
    end  
    jj = jj+1;
end    

if error==0
    % freq. range fmin_t - fmax_t is used as validity range of E_i and E_r.
    % amplitudes after decomposition (if h constant: abs(K)=1):
    a_inc = a(:,1).*K(1:length(f_LF),1);
    a_rfL = a(:,2).*conj(K(1:length(f_LF),1));

    % variance spectrum incident and reflected wave:
    amp_i       = abs(a_inc);
    amp_r       = abs(a_rfL);
    E_i         = Efact .* (amp_i.^2);
    E_r         = Efact .* (amp_r.^2);
                                                       
%============================= calculation of reflection ===============================
    % Hm0i, Hm0r calculated from E_i and E_r, fmin_t<=f<=fmax_t, 
    % where Hm0i = 4*m0i^0.5 and Hm0r = 4*m0r^0.5,
    % m0i: 0th moment of E_i, fmin_t<=f<=fmax_t,
    % m0r: 0th moment of E_r, fmin_t<=f<=fmax_t.
    Outref  = FN_reflec3(fmin_t, fmax_t, df, E_i, E_r);
    % copy freq-ranges to Outref:
    Outref.fmin_t   = fmin_t;
    Outref.fmax_t   = fmax_t;
    Outref.fmin_cr  = fmin_cr;
    Outref.fmax_cr  = fmax_cr;

    % sum of variances over bands df (= elementary freq. step):
    fact = df;
    Outref.sumi = fact*sum(E_i);
    Outref.sumr = fact*sum(E_r);
    if Outparmsp.sp_sum==0
        Fs      = f_LF;
        Es_i    = E_i;
        Es_r    = E_r;
    else    
        [Fs,Es_i]       = FN_sum7(f_LF, E_i, fres);
        [Fs,Es_r]       = FN_sum7(f_LF, E_r, fres);
    end    

    % peak periods of incident and reflected wave:
    % up till 14 sep 04: from fmin_cr to fmax_cr
    % changed on 14 sep 04: from fmin_t to fmax_t
    [Outref.Tpim,Outref.fpim,Outref.Epim] = ...
        FN_piekper8(fmin_t,fmax_t,Fs,Es_i);

    % reconstruction of incident and reflected wave:
    if outs(7)==1
        [y_inc,y_rfL] = FN_reconst(a_inc, a_rfL, f_LF, fmin_t, fmax_t, N);
    end    
end     % no errors in decomposition part
    
% end FN_decomp