function [fmin_js,fmax_js] = FN_jonswap(Tp,thresh,f_min,f_max);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   FN_Jonswap
%   calculate freq-range of ideal Jonswap spectrum and compare
%   the result with a freq-range specified by the user
%   version 1: 
%   26-Jan-04
%   a part of this script is copied from bctriton.m created 
%   by Ap van Dongeren and Jonswap.m by Henk Jan Bakkenes, 
%   WL|Delft Hydraulics
%
%   adapted to Decomp by: H. Klaasman, 31-Mar-05
%   in Decomp: the freq-range specified by the user is the
%   resulting freq-range from user specifications and thresh in
%   spectra from wave gauges (summed over f-bands). 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Input:
%   Tp:     peak period
%   thresh: E>=thresh*Emax
%   f_min, f_max:   freq-range specified by the user
%
%   Output:
%   [fmin_js,fmax_js]: freq-range where the variance density of a Jonswap
%   spectrum is below thresh*Emax.
%
%
% fm : peak frequency [Hz]
% N : number of wave components (not found here, Hkl, 21-12-04) 
% dt : time step (not found here, Hkl, 21-12-04) 


%JONSWAP parameters
df=0.01;
f=df:df:4/Tp;  % bovengrens f-gebied was 4 Hz, aangepast, afh. v. Tp
                % Hkl, 5-1-05
            % voor zeer lage drempel kan f_max hoger liggen, maar
            % waarschijnlijk is er bij hoge frequenties alleen ruis
Hm0gew  = 1; % alleen gebruikt voor normering, Hkl, 21-12-04
fm      = 1/Tp; % dit aanpassen
%lower   = floor(0.5423/df); % dit aanpassen
%upper   = floor(1.5303/df); % dit aanpassen


% jonswap maken 
% (waarschijnlijk is dit deel van Ap, Hkl, 21-12-04)
alpha= 0.0081 ;
gamma0 = 3.3;
g=9.81;
siga=0.07;
sigb=0.09;
ifm = floor(fm/df);
gamma1(1:ifm) = gamma0.^(exp(-.5*((f(1:ifm)-fm)./(siga*fm)).^2));
gamma1(ifm+1:length(f)) = ...
   gamma0.^(exp(-.5*((f(ifm+1:length(f))-fm)./(sigb*fm)).^2));
E = alpha*(g^2)*((2*pi)^(-4))*f.^(-5).* ...
   exp(-1.25*(f/fm).^(-4)).*gamma1;
Hm0temp = 4*sqrt(sum(E)*df);

E = E*(Hm0gew/Hm0temp)^2;

% calculate region of interest (part of Hkl)
% bepaal gebied waarin E>=thresh*max(E):
%drempel = thresh*sum(E)/nf;

% calculate index of peak in E:
[Ep,ip] = max(E);

% calculate threshold:
drempel = thresh*max(E);

i   = ip;
nf  = length(f);
while E(i)>=drempel & i<nf
    i = i+1;
end
fmax_js = f(i);

i = ip;
while E(i)>=drempel & i>1
    i = i-1;
end
fmin_js = f(i);

% warn if freq-range outside range specified by user:
if fmin_js<0.95*f_min | fmax_js>1.05*f_max
    tekst1 = sprintf('\n**** Warning ****\n\n');
    tekst2 = sprintf('Tp = %6.2f s\n',Tp);
    tekst3 = sprintf...
        ('freq-range of ideal Jonswap-spectrum: %6.2f - %6.2f Hz\n', ...
        fmin_js, fmax_js);
    tekst = [tekst1 tekst2 tekst3];
%    tekst4 = sprintf('change specifications or threshold');
%    tekst = [tekst1 tekst2 tekst3 tekst4];
    h=msgbox(tekst);
    uiwait(h);
end    

% end FN_jonswap