function [F,A_jjpp,Outparm] = FN_spD(in_naam,Ts,A2,Inparm,Scanparm,outs)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% FN_spD
%   Calculate spectra and spectral moments of matrix A2 with time 
%   vector Ts
%   version 1
%   26-Jan-05, changed: 26-May-05, eps->eps1
%
%
%
%               H. Klaasman
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   input: 
%   Ts: time vector
%   A1: matrix of scaled time signals per channel
%	Inparm:	structure met invoerparameters
%       dt:     tijdstap
%       fres:   gewenste frequentieresolutie in variantiespectrum
%       f_max:  bovengrens te berekenen freq.gebied
%       thresh: voor Spect: drempel voor het te plotten deel van het spectrum
%       colt:   kanaal met de tijd (0 voor geen)
%       ncols:  aantal te verwerken kanalen
%	    col:	rijvector met kolomnummers kanalen
%       xp:     rijvector met X-coordinaten (xp: xpos, n.a.v. Decomp v.14 en ouder)
%       hxp:    dieptes op plaatsen van de golfhoogtemeters (wordt berekend
%               uit overige invoerparameters, alleen Decomp)
%       x:      rijvector met x-coordinaten waarvoor het diepteprofiel is
%               gegeven
%       h:      dieptes in diepteprofiel
%   Scanparm:   structure gegevens over met gekozen samples:
%       tin1:   gevraagde begintijd 
%       tin2:   gevraagde eindtijd
%       step:   gevraagde monsterstap bij selectie
%       tuit1:  begintijd na bemonsteren
%       tuit2:  eindtijd na bemonsteren
%       dt:     tijdstap na bemonsteren
%   outs:   parameter indicating which spectra to be plotted:
%           raw spectra:        out(1)=1: plot, 0: don't plot
%           filtered spectra:   out(2)=1: plot, 0: don't plot
%   output:
%   F:      frequency vector of the raw spectrum
%   A_jjpp: matrix of FFT-values per chan.
        % removed, not used later:
        %   Fs:     frequency vector of variance spectra averaged
        %           over freq.bands of specified length (if possible)
        %   Es:     matrix of averaged variance spectra per chan.
%   Outparm:  structure with spectral parameters
%               (calculated from raw spectra)
%       N:      number of samples used in analyses     
%       df:     frequency step in raw spectra
%       fres:   frequency resolution used to calulate av. spectra
%       sp_sum: 1: spectra will be summed over bands,
%               0: Inparm.fres<df, no summation, fres set to df
%       Tpm:    vector of peak periods
%       fpm:    vector of peak frequencies
%       Epm:    variance density at peak frequency
%       f_min, f_max:   same as inparm if Inparm.f_max>0, otherwise
%               f_max calculated from criterium
%       fmin_s, fmax_s: connected region of E>=thresh*Emax
%       fmin_r, fmax_r: region of calculation spectral moments (combination
%                       of f_min(max) and fmin(max)_s)
%       m:      matrix of spectral moments, m(i,j) = ith moment of ch j
%       T0:     matrix of peak periods, T(1,j): Tp of ch j, 
%               T(2,j): T80% of ch j
%       eps, kappa, gamma: combinations of moments, described in manual
%       Hm0     = 4.*(m(1,:).^0.5);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%                    
        
% bandwidth in avaraged variance spectra:
fres    = Inparm.fres;  % desired frequency resolution * Tp
thresh  = Inparm.thresh;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
            % if a fixed threshold is whished, specify here:
            % thresh           = 0.005;
            % Inparm.thresh   = thresh;
      

% frequency range
f_min   = Inparm.f_min;
f_max   = Inparm.f_max;

% automatical detection (2 gauges) or default 2 or more gauges):
if Inparm.f_max<0
    [f_min,f_max] = FN_freqs(Inparm.xp,Inparm.x,Inparm.h);
end    
           
    
%t = Ts;

% standard settings:
[nsamp,nchan]   = size(A2);
no_gauges       = nchan;
dt              = Scanparm.dt;      % dt: time step                    
N               = length(Ts);       % N	 : number of data points [-]
T               = N*dt;          	% T  : total measurement duration [s]    
df              = 1/T;				% df : frequency step [Hz]
                        
% correct resolution if needed:
if fres<2*df
    fres    = df;
    sp_sum  = 0;    % the spectra will not be summed
else
    sp_sum  = 1;
end    
Outparm.N     = N;
Outparm.fres  = fres;
Outparm.df    = df;
Outparm.sp_sum  = sp_sum;
   
% the next two lines have been moved to enable the calcution
% of spectral moments of the original signal, according
% to Reflecmf of Delft Hydraulics
%                    f_LF        = [df:df:f_max]';  	% f_LF : LF frequency vector
%                    K           = phase_int(xp,f_LF,x,h);  
%                                  % determination of wave phase and shoaling

% =================================================================================
% ========== FFT of A2 ========================================================
% =================================================================================
                          

% ================= SCALE ===================================================
% The FFT in Matlab is defined as
%   X(k) = sum(n=1:N) (x(n) exp(-j*2pi(k-1)*(n-1)/N)),   1<=k<=N
% where
%   x(n) = 1/N sum(k=1:N) (X(k) exp(j*2pi(k-1)*(n-1)/N)), 1<=n<=N.
% If amplitudes are needed:
%   x(n) = sum(k=1:N)(c(k) * exp(j*2pi(k-1)*(n-1)/N)),    1<=n<=N.
% resulting in: 1/N*X(k) = c(k) = 0.5 * A_temp(k)  
% (A_temp is calculated in the program below).
% The factor 0.5 is added since one half of the spectrum will be used 
% (x(n) is real, the spectrum is symmetric).
%
% The variance spectrum can be approximated from the squared modulus of the 
% amplitudespectrum.
% If x{n}, 1<=n<=N is a stochastic signal, the next equation holds:
%   E{|c(k)|^2} = integr(fk-df/2:fk+df/2) E0(f) df,
% in which E0 is a two-sided spectrum. The moment m0 can be approximated by:
%   m0 = sum(k=1:N) (abs(c(k))^2) = 1/4 sum(k=1:N) (abs(A_temp(k))^2)
% m0 over the total frequency range must equal the variance of the time signal.
% E0(fk) can be approximated by |c(k))|^2 devided by the interval df=1/Ndt:
%   E0 = (1/4)*Ndt * abs(A_temp).^2.
% The one-sided spectrum: E = 2E0
% ================= SCALE =====================================================

Efact   = 0.5*N*dt;
A_jjpp  = zeros(N-1,no_gauges);
m       = zeros(4,no_gauges);
Tp      = zeros(2,no_gauges);
fp      = zeros(2,no_gauges);
Ep      = zeros(1,no_gauges);
Tpm     = zeros(2,no_gauges);
fpm     = zeros(2,no_gauges);
Epm     = zeros(1,no_gauges);
T0p     = zeros(3,no_gauges);
epsp    = zeros(2,no_gauges);

% amplitude and variance spectra of wave gauges:
nfig=0;
for pp = 1:no_gauges
     A_temp = 2/N*fft(A2(:,pp));  	% Fourier transform wave signal for 
                                        % all stations
     A_temp(1) = [];	           % Delete average value in FFT

     % variantiespectrum:
     nf      = round(length(A_temp)/2);
     F       = [df:df:(nf*df)]';
     E_temp  = Efact .* (abs(A_temp(1:nf)) .^ 2);
                        
     % average variance spectum over frequncy bands:
     if sp_sum==0
         Fs = F;
         Es = E_temp;
     else
         [Fs,Es] = FN_sum7(F, E_temp, fres);
     end    
                        
     % peak periods Tp and TpD from averaged variance spectrum
     %                      
     [Tpp,fpp,Epp,fminspp,fmaxspp] = FN_piekper4(Fs,Es,thresh);
     Tpm(:,pp)   = Tpp';
     fpm(:,pp)   = fpp';
     Epm(pp)     = Epp;
     fminss(:,pp) = fminspp;
     fmaxss(:,pp) = fmaxspp;

     % plot raw spectrum:
     if outs(1)==1
         nfig = nfig+1;
         figure(nfig);
         plot(F(1:2500),E_temp(1:2500));
     end
     % plot averaged spectrum:
     if outs(2)==1
         method  = 1;
         ch      = Inparm.col(pp);
         nfig    = nfig+1;
         FN_plotD12(nfig,in_naam,ch,Fs,Es,fmaxspp,fres,...
                   thresh,method);
     end    
             
     % the next line has been commented out, since spectral moments
     % of the original time signal will be determined:
%                           A_temp(length(f_LF)+1:size(A_temp,1)) = [];  
                                                    % Only necessary for f_LF  
      A_jjpp(:,pp) = A_temp;
      E_jjpp(:,pp) = E_temp;
end    
                    
clear A_temp;
clear E_temp;
clear Es;

% resulting peak periods and peak frequencies:
Outparm.Tpm     = Tpm;
Outparm.fpm     = fpm;
Outparm.Epm     = Epm;

% connected range where spectrum >thresh
fmin_s          = max(fminss);
fmax_s          = min(fmaxss);
Outparm.f_min   = f_min;
Outparm.f_max   = f_max;
Outparm.fmin_s  = fmin_s;
Outparm.fmax_s  = fmax_s;
% range to be used in calculation of statistical parameters 
% from spectrum:
fmin_r          = fmin_s;
fmax_r          = fmax_s;
Outparm.fmin_r  = fmin_r;
Outparm.fmax_r  = fmax_r;
                 
% moments from variance spectrum:
m = FN_mom6(F,E_jjpp,fmin_r,fmax_r);

% variables derived from spectrum:
for pp=1:no_gauges
     [T0pp,epspp,kappapp,gammapp] = ...
          FN_wavvars3(F,E_jjpp(:,pp),m(:,pp),fmin_r,fmax_r);
     T0(:,pp) = T0pp;
     eps1(:,pp)= epspp;
     kappa(pp) = kappapp;
     gamma(pp) = gammapp;
end   
                        
Outparm.m       = m;
Outparm.T0      = T0;
Outparm.eps     = eps1;
Outparm.kappa   = kappa;
Outparm.gamma   = gamma;
Outparm.Hm0     = 4.*(m(1,:).^0.5);
Outparm.fmin_s  = fmin_s;
Outparm.fmax_s  = fmax_s;
                   
% frequency range used to calculate statistical parameters
% from incident and reflected:
fmin_t = max(f_min,fmin_s);
fmax_t = min(f_max,fmax_s);
%     fmin_t = f_min;
%     fmax_t = f_max;
Outparm.fmin_t = fmin_t;
Outparm.fmax_t = fmax_t;
                   
% end FN_spD