function A = FN_testsig73(Ag,Inparm)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% FN_testsig73
% maakt een testsignaal met een Jonswapspectrum uit een
% stuursignaal gemaakt door Multilin voor het bepalen van de
% terugkaatsingscoefficient van een onregelmatige golf
%   versie 3, met dispersie
%   21-11-03, aanpassing: 2-2-05
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   versie 2: zonder dispersie, c=c(Tp)
% 						
%                                       H. Klaasman
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Invoer:
%   Ag:    golfvector
%	Inparm:	structure met invoerparameters:
%       Tp	    piekperiode
%       Hm0     parmameter voor Multilin, ~ significate golfhoogte
%       dt	    tijdstap
%       h		waterdiepte
%       Kr	    veronderstelde terugkaatsingscoefficient
%       x	    vector met veronderstelde posities golfhoogtemeters
%
% Uitvoer:
%   A:      A(i,j)=resulterende golf op tijdstip i*dt en plaats x(j)
%
%   methode: stel Bg=FFT(Ag)
%   per spectrale term j kan hieruit een resulterende golf B worden
%   berekend, met
%	B(j,p) = Bg(j)exp(-i*phi1(j,p)) + Kr*Bg(j)exp(-i*phi2(j,p)).
%   Hierin is:
%   phi1(j,p)   = (x(p)-x(1))*k(j)
%   phi2(j,p)   = (2(len-x(1))-x(p))*k(j)
%   k(j)        = dispers(h,w(j)), w(j) = 2pi*j/lentgh(Ag)*dt 
%   len: lengte goot = 40 m
% let op:
%   a=Ag, a(1) is het oudste; golfhoogtemeters verder weg (grotere x
%   bevatten een ouder gedeelte van de golf (kleinere i); de terug-
%   gekaatste golf bevat een ouder gedeelte dan de invallende. 
%   De tijdverschuiving als functie van de plaats is:
%   a(x,t) = a(0,t-ts), ts = x/c = x*Tp/Lp voor de piekperiode;
%   voor alle andere periodes in het Jonswapspectrum geldt een
%   andere voortplantingssnelheid c, en dus een andere T/L. 
%   Voorlopig wordt c constant verondersteld.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Tp      = Inparm.Tp;
dt      = Inparm.dt;
h       = Inparm.h;  
Kr      = Inparm.Kr;
x       = Inparm.x;
len     = 40;

nx      = length(x);
nsamp   = length(Ag);
T       = nsamp*dt;

% bepaal FFT:
Bg      = fft(Ag);
Bg(1)   = [];
nf      = length(Bg);
nf1     = round(nf/2)
Bg(nf1+1:nf)= [];
df      = 1/T;
f       = df:df:nf1*df;
f       = f';

% berekening faseverschuivingen
w       = 2*pi*f;
g       = 9.81;
k       = DISPER(w,h,g);  % k: vector, wordt 2D als h niet constant
e       = ones(1,nx);
dx1     = x-x(1)*e;
dx2     = dx1 + 2*(len*e-x);
phi1    = k*dx1;    % matrix met k(j)*dx1(k)
phi2    = k*dx2;
 
% resulterende oppervlakte-uitwijkingen:
nf2 = 2*nf1+1;
A   = zeros(nf2,nx);
i = (-1)^0.5;
for p=1:nx
    B       = Bg.*exp(-i*phi1(:,p)) + (Kr*Bg).*exp(-i*phi2(:,p));
    B1      = [0 ; B ; flipud(conj(B))];
    A(:,p)  = real(ifft(B1));
end

for p=1:nx
   subplot(nx,1,p);
   plot(A(1:500,p));
end    

% einde FN_testsig73